/*
 * This file was generated by orbit-idl-2 - DO NOT EDIT!
 */

#include <string.h>
#define ORBIT2_STUBS_API
#include "evolution-calendar.h"

void GNOME_Evolution_Calendar_CalView_start(GNOME_Evolution_Calendar_CalView _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalView__iinterface.methods, 0, NULL, NULL, NULL, ev, GNOME_Evolution_Calendar_CalView__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalView__epv, start),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalView_start);

}
void GNOME_Evolution_Calendar_CalViewListener_notifyObjectsAdded(GNOME_Evolution_Calendar_CalViewListener _obj, const GNOME_Evolution_Calendar_stringlist* objects, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)objects;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalViewListener__iinterface.methods, 0, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalViewListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalViewListener__epv, notifyObjectsAdded),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyObjectsAdded);

}
void GNOME_Evolution_Calendar_CalViewListener_notifyObjectsModified(GNOME_Evolution_Calendar_CalViewListener _obj, const GNOME_Evolution_Calendar_stringlist* objects, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)objects;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalViewListener__iinterface.methods, 1, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalViewListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalViewListener__epv, notifyObjectsModified),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyObjectsModified);

}
void GNOME_Evolution_Calendar_CalViewListener_notifyObjectsRemoved(GNOME_Evolution_Calendar_CalViewListener _obj, const GNOME_Evolution_Calendar_CalObjIDSeq* ids, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)ids;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalViewListener__iinterface.methods, 2, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalViewListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalViewListener__epv, notifyObjectsRemoved),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyObjectsRemoved);

}
void GNOME_Evolution_Calendar_CalViewListener_notifyQueryProgress(GNOME_Evolution_Calendar_CalViewListener _obj, const CORBA_char * message, const CORBA_short percent, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&message;
_args[1] = (gpointer)&percent;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalViewListener__iinterface.methods, 3, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalViewListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalViewListener__epv, notifyQueryProgress),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyQueryProgress);

}
void GNOME_Evolution_Calendar_CalViewListener_notifyQueryDone(GNOME_Evolution_Calendar_CalViewListener _obj, const GNOME_Evolution_Calendar_CallStatus status, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&status;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalViewListener__iinterface.methods, 4, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalViewListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalViewListener__epv, notifyQueryDone),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyQueryDone);

}
CORBA_string GNOME_Evolution_Calendar_Cal__get_uri(GNOME_Evolution_Calendar_Cal _obj, CORBA_Environment *ev){
CORBA_string _ORBIT_retval;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 0, &_ORBIT_retval, NULL, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, _get_uri),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal__get_uri);

return _ORBIT_retval;
}
void GNOME_Evolution_Calendar_Cal_open(GNOME_Evolution_Calendar_Cal _obj, const CORBA_boolean only_if_exists, const CORBA_char * username, const CORBA_char * password, CORBA_Environment *ev){
gpointer _args[3];
_args[0] = (gpointer)&only_if_exists;
_args[1] = (gpointer)&username;
_args[2] = (gpointer)&password;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 1, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, open),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_open);

}
void GNOME_Evolution_Calendar_Cal_remove(GNOME_Evolution_Calendar_Cal _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 2, NULL, NULL, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, remove),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_remove);

}
void GNOME_Evolution_Calendar_Cal_isReadOnly(GNOME_Evolution_Calendar_Cal _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 3, NULL, NULL, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, isReadOnly),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_isReadOnly);

}
void GNOME_Evolution_Calendar_Cal_getStaticCapabilities(GNOME_Evolution_Calendar_Cal _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 4, NULL, NULL, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getStaticCapabilities),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getStaticCapabilities);

}
void GNOME_Evolution_Calendar_Cal_getCalAddress(GNOME_Evolution_Calendar_Cal _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 5, NULL, NULL, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getCalAddress),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getCalAddress);

}
void GNOME_Evolution_Calendar_Cal_getAlarmEmailAddress(GNOME_Evolution_Calendar_Cal _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 6, NULL, NULL, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getAlarmEmailAddress),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getAlarmEmailAddress);

}
void GNOME_Evolution_Calendar_Cal_getLdapAttribute(GNOME_Evolution_Calendar_Cal _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 7, NULL, NULL, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getLdapAttribute),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getLdapAttribute);

}
void GNOME_Evolution_Calendar_Cal_setMode(GNOME_Evolution_Calendar_Cal _obj, const GNOME_Evolution_Calendar_CalMode mode, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&mode;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 8, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, setMode),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_setMode);

}
void GNOME_Evolution_Calendar_Cal_getDefaultObject(GNOME_Evolution_Calendar_Cal _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 9, NULL, NULL, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getDefaultObject),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getDefaultObject);

}
void GNOME_Evolution_Calendar_Cal_getObject(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * uid, const CORBA_char * rid, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&uid;
_args[1] = (gpointer)&rid;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 10, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getObject),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getObject);

}
void GNOME_Evolution_Calendar_Cal_getObjectList(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * query, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&query;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 11, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getObjectList),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getObjectList);

}
void GNOME_Evolution_Calendar_Cal_getTimezone(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * tzid, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&tzid;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 12, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getTimezone),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getTimezone);

}
void GNOME_Evolution_Calendar_Cal_addTimezone(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * tz, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&tz;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 13, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, addTimezone),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_addTimezone);

}
void GNOME_Evolution_Calendar_Cal_setDefaultTimezone(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * tz, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&tz;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 14, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, setDefaultTimezone),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_setDefaultTimezone);

}
void GNOME_Evolution_Calendar_Cal_getChanges(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * change_id, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&change_id;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 15, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getChanges),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getChanges);

}
void GNOME_Evolution_Calendar_Cal_getFreeBusy(GNOME_Evolution_Calendar_Cal _obj, const GNOME_Evolution_Calendar_UserList* users, const GNOME_Evolution_Calendar_Time_t start, const GNOME_Evolution_Calendar_Time_t end, CORBA_Environment *ev){
gpointer _args[3];
_args[0] = (gpointer)users;
_args[1] = (gpointer)&start;
_args[2] = (gpointer)&end;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 16, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getFreeBusy),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getFreeBusy);

}
void GNOME_Evolution_Calendar_Cal_discardAlarm(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * uid, const CORBA_char * auid, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&uid;
_args[1] = (gpointer)&auid;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 17, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, discardAlarm),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_discardAlarm);

}
void GNOME_Evolution_Calendar_Cal_createObject(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * calobj, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&calobj;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 18, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, createObject),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_createObject);

}
void GNOME_Evolution_Calendar_Cal_modifyObject(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * calobj, const GNOME_Evolution_Calendar_CalObjModType mod, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&calobj;
_args[1] = (gpointer)&mod;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 19, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, modifyObject),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_modifyObject);

}
void GNOME_Evolution_Calendar_Cal_removeObject(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * uid, const CORBA_char * rid, const GNOME_Evolution_Calendar_CalObjModType mod, CORBA_Environment *ev){
gpointer _args[3];
_args[0] = (gpointer)&uid;
_args[1] = (gpointer)&rid;
_args[2] = (gpointer)&mod;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 20, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, removeObject),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_removeObject);

}
void GNOME_Evolution_Calendar_Cal_getAttachmentList(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * uid, const CORBA_char * rid, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&uid;
_args[1] = (gpointer)&rid;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 21, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getAttachmentList),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getAttachmentList);

}
void GNOME_Evolution_Calendar_Cal_receiveObjects(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * calobj, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&calobj;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 22, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, receiveObjects),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_receiveObjects);

}
void GNOME_Evolution_Calendar_Cal_sendObjects(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * calobj, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&calobj;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 23, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, sendObjects),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_sendObjects);

}
void GNOME_Evolution_Calendar_Cal_getQuery(GNOME_Evolution_Calendar_Cal _obj, const CORBA_char * sexp, const GNOME_Evolution_Calendar_CalViewListener ql, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&sexp;
_args[1] = (gpointer)&ql;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_Cal__iinterface.methods, 24, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_Cal__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_Cal__epv, getQuery),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getQuery);

}
void GNOME_Evolution_Calendar_CalListener_notifyReadOnly(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_boolean read_only, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&read_only;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 0, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyReadOnly),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyReadOnly);

}
void GNOME_Evolution_Calendar_CalListener_notifyCalAddress(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_char * address, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&address;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 1, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyCalAddress),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyCalAddress);

}
void GNOME_Evolution_Calendar_CalListener_notifyAlarmEmailAddress(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_char * address, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&address;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 2, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyAlarmEmailAddress),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyAlarmEmailAddress);

}
void GNOME_Evolution_Calendar_CalListener_notifyLDAPAttribute(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_char * ldap_attribute, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&ldap_attribute;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 3, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyLDAPAttribute),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyLDAPAttribute);

}
void GNOME_Evolution_Calendar_CalListener_notifyStaticCapabilities(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_char * capabilities, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&capabilities;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 4, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyStaticCapabilities),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyStaticCapabilities);

}
void GNOME_Evolution_Calendar_CalListener_notifyCalOpened(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&status;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 5, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyCalOpened),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyCalOpened);

}
void GNOME_Evolution_Calendar_CalListener_notifyCalRemoved(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&status;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 6, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyCalRemoved),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyCalRemoved);

}
void GNOME_Evolution_Calendar_CalListener_notifyObjectCreated(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_char * uid, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&uid;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 7, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyObjectCreated),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectCreated);

}
void GNOME_Evolution_Calendar_CalListener_notifyObjectModified(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&status;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 8, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyObjectModified),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectModified);

}
void GNOME_Evolution_Calendar_CalListener_notifyObjectRemoved(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&status;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 9, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyObjectRemoved),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectRemoved);

}
void GNOME_Evolution_Calendar_CalListener_notifyAlarmDiscarded(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&status;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 10, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyAlarmDiscarded),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyAlarmDiscarded);

}
void GNOME_Evolution_Calendar_CalListener_notifyObjectsReceived(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&status;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 11, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyObjectsReceived),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectsReceived);

}
void GNOME_Evolution_Calendar_CalListener_notifyObjectsSent(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const GNOME_Evolution_Calendar_UserList* users, const CORBA_char * calobj, CORBA_Environment *ev){
gpointer _args[3];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)users;
_args[2] = (gpointer)&calobj;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 12, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyObjectsSent),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectsSent);

}
void GNOME_Evolution_Calendar_CalListener_notifyDefaultObjectRequested(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_char * object, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&object;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 13, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyDefaultObjectRequested),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyDefaultObjectRequested);

}
void GNOME_Evolution_Calendar_CalListener_notifyObjectRequested(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_char * object, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&object;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 14, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyObjectRequested),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectRequested);

}
void GNOME_Evolution_Calendar_CalListener_notifyObjectListRequested(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const GNOME_Evolution_Calendar_stringlist* objects, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)objects;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 15, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyObjectListRequested),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectListRequested);

}
void GNOME_Evolution_Calendar_CalListener_notifyQuery(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const GNOME_Evolution_Calendar_CalView query, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&query;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 16, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyQuery),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyQuery);

}
void GNOME_Evolution_Calendar_CalListener_notifyTimezoneRequested(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_char * tz, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&tz;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 17, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyTimezoneRequested),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyTimezoneRequested);

}
void GNOME_Evolution_Calendar_CalListener_notifyTimezoneAdded(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const CORBA_char * tzid, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&tzid;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 18, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyTimezoneAdded),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyTimezoneAdded);

}
void GNOME_Evolution_Calendar_CalListener_notifyDefaultTimezoneSet(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&status;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 19, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyDefaultTimezoneSet),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyDefaultTimezoneSet);

}
void GNOME_Evolution_Calendar_CalListener_notifyChanges(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const GNOME_Evolution_Calendar_CalObjChangeSeq* changes, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)changes;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 20, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyChanges),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyChanges);

}
void GNOME_Evolution_Calendar_CalListener_notifyFreeBusy(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const GNOME_Evolution_Calendar_CalObjSeq* freebusy, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)freebusy;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 21, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyFreeBusy),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyFreeBusy);

}
void GNOME_Evolution_Calendar_CalListener_notifyAttachmentListRequested(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CallStatus status, const GNOME_Evolution_Calendar_stringlist* attachments, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)attachments;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 22, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyAttachmentListRequested),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyAttachmentListRequested);

}
void GNOME_Evolution_Calendar_CalListener_notifyCalSetMode(GNOME_Evolution_Calendar_CalListener _obj, const GNOME_Evolution_Calendar_CalListener_SetModeStatus status, const GNOME_Evolution_Calendar_CalMode mode, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&status;
_args[1] = (gpointer)&mode;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 23, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyCalSetMode),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyCalSetMode);

}
void GNOME_Evolution_Calendar_CalListener_notifyErrorOccurred(GNOME_Evolution_Calendar_CalListener _obj, const CORBA_char * message, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&message;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 24, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyErrorOccurred),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyErrorOccurred);

}
void GNOME_Evolution_Calendar_CalListener_notifyAuthRequired(GNOME_Evolution_Calendar_CalListener _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalListener__iinterface.methods, 25, NULL, NULL, NULL, ev, GNOME_Evolution_Calendar_CalListener__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__epv, notifyAuthRequired),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyAuthRequired);

}
GNOME_Evolution_Calendar_Cal GNOME_Evolution_Calendar_CalFactory_getCal(GNOME_Evolution_Calendar_CalFactory _obj, const CORBA_char * source, const GNOME_Evolution_Calendar_CalObjType type, const GNOME_Evolution_Calendar_CalListener listener, CORBA_Environment *ev){
GNOME_Evolution_Calendar_Cal _ORBIT_retval;
gpointer _args[3];
_args[0] = (gpointer)&source;
_args[1] = (gpointer)&type;
_args[2] = (gpointer)&listener;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CalFactory__iinterface.methods, 0, &_ORBIT_retval, _args, NULL, ev, GNOME_Evolution_Calendar_CalFactory__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CalFactory__epv, getCal),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CalFactory_getCal);

return _ORBIT_retval;
}
void GNOME_Evolution_Calendar_CompEditorFactory_editExisting(GNOME_Evolution_Calendar_CompEditorFactory _obj, const CORBA_char * uri, const CORBA_char * uid, const GNOME_Evolution_Calendar_CompEditorFactory_CompEditorMode mode, CORBA_Environment *ev){
gpointer _args[3];
_args[0] = (gpointer)&uri;
_args[1] = (gpointer)&uid;
_args[2] = (gpointer)&mode;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CompEditorFactory__iinterface.methods, 0, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CompEditorFactory__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CompEditorFactory__epv, editExisting),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CompEditorFactory_editExisting);

}
void GNOME_Evolution_Calendar_CompEditorFactory_editNew(GNOME_Evolution_Calendar_CompEditorFactory _obj, const CORBA_char * uri, const GNOME_Evolution_Calendar_CompEditorFactory_CompEditorMode mode, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&uri;
_args[1] = (gpointer)&mode;
ORBit_c_stub_invoke (_obj, &GNOME_Evolution_Calendar_CompEditorFactory__iinterface.methods, 1, NULL, _args, NULL, ev, GNOME_Evolution_Calendar_CompEditorFactory__classid, G_STRUCT_OFFSET (POA_GNOME_Evolution_Calendar_CompEditorFactory__epv, editNew),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Evolution_Calendar_CompEditorFactory_editNew);

}
