/*
 * This file was generated by orbit-idl-2 - DO NOT EDIT!
 */

#include <string.h>
#define ORBIT2_STUBS_API
#include "evolution-calendar.h"

static ORBitSmallSkeleton get_skel_small_GNOME_Evolution_Calendar_CalView(POA_GNOME_Evolution_Calendar_CalView *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 's':
if(strcmp((opname + 1), "tart")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalView_epv->start;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalView__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalView_start;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_GNOME_Evolution_Calendar_CalView__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_GNOME_Evolution_Calendar_CalView, "IDL:GNOME/Evolution/Calendar/CalView:1.0", &GNOME_Evolution_Calendar_CalView__classid, NULL, &GNOME_Evolution_Calendar_CalView__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_GNOME_Evolution_Calendar_CalView__fini,
   ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CalView__vepv, GNOME_Evolution_Calendar_CalView_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CalView__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_GNOME_Evolution_Calendar_CalView__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_GNOME_Evolution_Calendar_CalViewListener(POA_GNOME_Evolution_Calendar_CalViewListener *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'n':
switch(opname[1]) {
case 'o':
switch(opname[2]) {
case 't':
switch(opname[3]) {
case 'i':
switch(opname[4]) {
case 'f':
switch(opname[5]) {
case 'y':
switch(opname[6]) {
case 'O':
switch(opname[7]) {
case 'b':
switch(opname[8]) {
case 'j':
switch(opname[9]) {
case 'e':
switch(opname[10]) {
case 'c':
switch(opname[11]) {
case 't':
switch(opname[12]) {
case 's':
switch(opname[13]) {
case 'A':
if(strcmp((opname + 14), "dded")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalViewListener_epv->notifyObjectsAdded;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalViewListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyObjectsAdded;
break;
case 'M':
if(strcmp((opname + 14), "odified")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalViewListener_epv->notifyObjectsModified;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalViewListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyObjectsModified;
break;
case 'R':
if(strcmp((opname + 14), "emoved")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalViewListener_epv->notifyObjectsRemoved;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalViewListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyObjectsRemoved;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'Q':
switch(opname[7]) {
case 'u':
switch(opname[8]) {
case 'e':
switch(opname[9]) {
case 'r':
switch(opname[10]) {
case 'y':
switch(opname[11]) {
case 'D':
if(strcmp((opname + 12), "one")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalViewListener_epv->notifyQueryDone;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalViewListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [4];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyQueryDone;
break;
case 'P':
if(strcmp((opname + 12), "rogress")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalViewListener_epv->notifyQueryProgress;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalViewListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [3];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalViewListener_notifyQueryProgress;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_GNOME_Evolution_Calendar_CalViewListener__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_GNOME_Evolution_Calendar_CalViewListener, "IDL:GNOME/Evolution/Calendar/CalViewListener:1.0", &GNOME_Evolution_Calendar_CalViewListener__classid, NULL, &GNOME_Evolution_Calendar_CalViewListener__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_GNOME_Evolution_Calendar_CalViewListener__fini,
   ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CalViewListener__vepv, GNOME_Evolution_Calendar_CalViewListener_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CalViewListener__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_GNOME_Evolution_Calendar_CalViewListener__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_GNOME_Evolution_Calendar_Cal(POA_GNOME_Evolution_Calendar_Cal *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case '_':
if(strcmp((opname + 1), "get_uri")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->_get_uri;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal__get_uri;
break;
case 'a':
if(strcmp((opname + 1), "ddTimezone")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->addTimezone;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [13];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_addTimezone;
break;
case 'c':
if(strcmp((opname + 1), "reateObject")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->createObject;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [18];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_createObject;
break;
case 'd':
if(strcmp((opname + 1), "iscardAlarm")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->discardAlarm;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [17];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_discardAlarm;
break;
case 'g':
switch(opname[1]) {
case 'e':
switch(opname[2]) {
case 't':
switch(opname[3]) {
case 'A':
switch(opname[4]) {
case 'l':
if(strcmp((opname + 5), "armEmailAddress")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getAlarmEmailAddress;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [6];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getAlarmEmailAddress;
break;
case 't':
if(strcmp((opname + 5), "tachmentList")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getAttachmentList;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [21];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getAttachmentList;
break;
default: break; 
}
break;
case 'C':
switch(opname[4]) {
case 'a':
if(strcmp((opname + 5), "lAddress")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getCalAddress;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [5];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getCalAddress;
break;
case 'h':
if(strcmp((opname + 5), "anges")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getChanges;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [15];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getChanges;
break;
default: break; 
}
break;
case 'D':
if(strcmp((opname + 4), "efaultObject")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getDefaultObject;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [9];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getDefaultObject;
break;
case 'F':
if(strcmp((opname + 4), "reeBusy")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getFreeBusy;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [16];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getFreeBusy;
break;
case 'L':
if(strcmp((opname + 4), "dapAttribute")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getLdapAttribute;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [7];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getLdapAttribute;
break;
case 'O':
switch(opname[4]) {
case 'b':
switch(opname[5]) {
case 'j':
switch(opname[6]) {
case 'e':
switch(opname[7]) {
case 'c':
switch(opname[8]) {
case 't':
switch(opname[9]) {
case '\0':
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getObject;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [10];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getObject;
break;
case 'L':
if(strcmp((opname + 10), "ist")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getObjectList;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [11];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getObjectList;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'Q':
if(strcmp((opname + 4), "uery")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getQuery;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [24];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getQuery;
break;
case 'S':
if(strcmp((opname + 4), "taticCapabilities")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getStaticCapabilities;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [4];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getStaticCapabilities;
break;
case 'T':
if(strcmp((opname + 4), "imezone")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->getTimezone;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [12];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_getTimezone;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'i':
if(strcmp((opname + 1), "sReadOnly")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->isReadOnly;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [3];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_isReadOnly;
break;
case 'm':
if(strcmp((opname + 1), "odifyObject")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->modifyObject;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [19];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_modifyObject;
break;
case 'o':
if(strcmp((opname + 1), "pen")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->open;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_open;
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
switch(opname[1]) {
case 'e':
switch(opname[2]) {
case 'c':
if(strcmp((opname + 3), "eiveObjects")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->receiveObjects;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [22];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_receiveObjects;
break;
case 'f':
if(strcmp((opname + 3), "")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'm':
switch(opname[3]) {
case 'o':
switch(opname[4]) {
case 'v':
switch(opname[5]) {
case 'e':
switch(opname[6]) {
case '\0':
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->remove;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_remove;
break;
case 'O':
if(strcmp((opname + 7), "bject")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->removeObject;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [20];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_removeObject;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 's':
switch(opname[1]) {
case 'e':
switch(opname[2]) {
case 'n':
if(strcmp((opname + 3), "dObjects")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->sendObjects;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [23];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_sendObjects;
break;
case 't':
switch(opname[3]) {
case 'D':
if(strcmp((opname + 4), "efaultTimezone")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->setDefaultTimezone;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [14];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_setDefaultTimezone;
break;
case 'M':
if(strcmp((opname + 4), "ode")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_Cal_epv->setMode;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_Cal__iinterface;*m_data = (gpointer)&_t_->methods._buffer [8];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_Cal_setMode;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_GNOME_Evolution_Calendar_Cal__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_GNOME_Evolution_Calendar_Cal, "IDL:GNOME/Evolution/Calendar/Cal:1.0", &GNOME_Evolution_Calendar_Cal__classid, NULL, &GNOME_Evolution_Calendar_Cal__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_GNOME_Evolution_Calendar_Cal__fini,
   ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_Cal__vepv, GNOME_Evolution_Calendar_Cal_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_Cal__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_GNOME_Evolution_Calendar_Cal__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_GNOME_Evolution_Calendar_CalListener(POA_GNOME_Evolution_Calendar_CalListener *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'n':
switch(opname[1]) {
case 'o':
switch(opname[2]) {
case 't':
switch(opname[3]) {
case 'i':
switch(opname[4]) {
case 'f':
switch(opname[5]) {
case 'y':
switch(opname[6]) {
case 'A':
switch(opname[7]) {
case 'l':
switch(opname[8]) {
case 'a':
switch(opname[9]) {
case 'r':
switch(opname[10]) {
case 'm':
switch(opname[11]) {
case 'D':
if(strcmp((opname + 12), "iscarded")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyAlarmDiscarded;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [10];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyAlarmDiscarded;
break;
case 'E':
if(strcmp((opname + 12), "mailAddress")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyAlarmEmailAddress;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyAlarmEmailAddress;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 't':
if(strcmp((opname + 8), "tachmentListRequested")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyAttachmentListRequested;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [22];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyAttachmentListRequested;
break;
case 'u':
if(strcmp((opname + 8), "thRequired")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyAuthRequired;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [25];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyAuthRequired;
break;
default: break; 
}
break;
case 'C':
switch(opname[7]) {
case 'a':
switch(opname[8]) {
case 'l':
switch(opname[9]) {
case 'A':
if(strcmp((opname + 10), "ddress")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyCalAddress;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyCalAddress;
break;
case 'O':
if(strcmp((opname + 10), "pened")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyCalOpened;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [5];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyCalOpened;
break;
case 'R':
if(strcmp((opname + 10), "emoved")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyCalRemoved;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [6];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyCalRemoved;
break;
case 'S':
if(strcmp((opname + 10), "etMode")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyCalSetMode;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [23];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyCalSetMode;
break;
default: break; 
}
break;
default: break; 
}
break;
case 'h':
if(strcmp((opname + 8), "anges")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyChanges;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [20];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyChanges;
break;
default: break; 
}
break;
case 'D':
switch(opname[7]) {
case 'e':
switch(opname[8]) {
case 'f':
switch(opname[9]) {
case 'a':
switch(opname[10]) {
case 'u':
switch(opname[11]) {
case 'l':
switch(opname[12]) {
case 't':
switch(opname[13]) {
case 'O':
if(strcmp((opname + 14), "bjectRequested")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyDefaultObjectRequested;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [13];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyDefaultObjectRequested;
break;
case 'T':
if(strcmp((opname + 14), "imezoneSet")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyDefaultTimezoneSet;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [19];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyDefaultTimezoneSet;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'E':
if(strcmp((opname + 7), "rrorOccurred")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyErrorOccurred;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [24];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyErrorOccurred;
break;
case 'F':
if(strcmp((opname + 7), "reeBusy")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyFreeBusy;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [21];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyFreeBusy;
break;
case 'L':
if(strcmp((opname + 7), "DAPAttribute")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyLDAPAttribute;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [3];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyLDAPAttribute;
break;
case 'O':
switch(opname[7]) {
case 'b':
switch(opname[8]) {
case 'j':
switch(opname[9]) {
case 'e':
switch(opname[10]) {
case 'c':
switch(opname[11]) {
case 't':
switch(opname[12]) {
case 'C':
if(strcmp((opname + 13), "reated")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyObjectCreated;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [7];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectCreated;
break;
case 'L':
if(strcmp((opname + 13), "istRequested")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyObjectListRequested;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [15];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectListRequested;
break;
case 'M':
if(strcmp((opname + 13), "odified")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyObjectModified;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [8];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectModified;
break;
case 'R':
switch(opname[13]) {
case 'e':
switch(opname[14]) {
case 'm':
if(strcmp((opname + 15), "oved")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyObjectRemoved;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [9];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectRemoved;
break;
case 'q':
if(strcmp((opname + 15), "uested")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyObjectRequested;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [14];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectRequested;
break;
default: break; 
}
break;
default: break; 
}
break;
case 's':
switch(opname[13]) {
case 'R':
if(strcmp((opname + 14), "eceived")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyObjectsReceived;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [11];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectsReceived;
break;
case 'S':
if(strcmp((opname + 14), "ent")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyObjectsSent;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [12];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyObjectsSent;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'Q':
if(strcmp((opname + 7), "uery")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyQuery;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [16];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyQuery;
break;
case 'R':
if(strcmp((opname + 7), "eadOnly")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyReadOnly;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyReadOnly;
break;
case 'S':
if(strcmp((opname + 7), "taticCapabilities")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyStaticCapabilities;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [4];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyStaticCapabilities;
break;
case 'T':
switch(opname[7]) {
case 'i':
switch(opname[8]) {
case 'm':
switch(opname[9]) {
case 'e':
switch(opname[10]) {
case 'z':
switch(opname[11]) {
case 'o':
switch(opname[12]) {
case 'n':
switch(opname[13]) {
case 'e':
switch(opname[14]) {
case 'A':
if(strcmp((opname + 15), "dded")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyTimezoneAdded;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [18];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyTimezoneAdded;
break;
case 'R':
if(strcmp((opname + 15), "equested")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalListener_epv->notifyTimezoneRequested;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [17];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalListener_notifyTimezoneRequested;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_GNOME_Evolution_Calendar_CalListener__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_GNOME_Evolution_Calendar_CalListener, "IDL:GNOME/Evolution/Calendar/CalListener:1.0", &GNOME_Evolution_Calendar_CalListener__classid, NULL, &GNOME_Evolution_Calendar_CalListener__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_GNOME_Evolution_Calendar_CalListener__fini,
   ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__vepv, GNOME_Evolution_Calendar_CalListener_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CalListener__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_GNOME_Evolution_Calendar_CalListener__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_GNOME_Evolution_Calendar_CalFactory(POA_GNOME_Evolution_Calendar_CalFactory *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'g':
if(strcmp((opname + 1), "etCal")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CalFactory_epv->getCal;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CalFactory__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CalFactory_getCal;
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_GNOME_Evolution_Calendar_CalFactory__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_GNOME_Evolution_Calendar_CalFactory, "IDL:GNOME/Evolution/Calendar/CalFactory:1.0", &GNOME_Evolution_Calendar_CalFactory__classid, NULL, &GNOME_Evolution_Calendar_CalFactory__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_GNOME_Evolution_Calendar_CalFactory__fini,
   ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CalFactory__vepv, GNOME_Evolution_Calendar_CalFactory_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CalFactory__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_GNOME_Evolution_Calendar_CalFactory__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_GNOME_Evolution_Calendar_CompEditorFactory(POA_GNOME_Evolution_Calendar_CompEditorFactory *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'e':
switch(opname[1]) {
case 'd':
switch(opname[2]) {
case 'i':
switch(opname[3]) {
case 't':
switch(opname[4]) {
case 'E':
if(strcmp((opname + 5), "xisting")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CompEditorFactory_epv->editExisting;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CompEditorFactory__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CompEditorFactory_editExisting;
break;
case 'N':
if(strcmp((opname + 5), "ew")) break;
*impl = (gpointer)servant->vepv->GNOME_Evolution_Calendar_CompEditorFactory_epv->editNew;
{ORBit_IInterface *volatile _t_=&GNOME_Evolution_Calendar_CompEditorFactory__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_GNOME_Evolution_Calendar_CompEditorFactory_editNew;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_GNOME_Evolution_Calendar_CompEditorFactory__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_GNOME_Evolution_Calendar_CompEditorFactory, "IDL:GNOME/Evolution/Calendar/CompEditorFactory:1.0", &GNOME_Evolution_Calendar_CompEditorFactory__classid, NULL, &GNOME_Evolution_Calendar_CompEditorFactory__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_GNOME_Evolution_Calendar_CompEditorFactory__fini,
   ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CompEditorFactory__vepv, GNOME_Evolution_Calendar_CompEditorFactory_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_CompEditorFactory__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_GNOME_Evolution_Calendar_CompEditorFactory__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_GNOME_Evolution_Calendar_AlarmNotify(POA_GNOME_Evolution_Calendar_AlarmNotify *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_GNOME_Evolution_Calendar_AlarmNotify__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_GNOME_Evolution_Calendar_AlarmNotify, "IDL:GNOME/Evolution/Calendar/AlarmNotify:1.0", &GNOME_Evolution_Calendar_AlarmNotify__classid, NULL, &GNOME_Evolution_Calendar_AlarmNotify__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_GNOME_Evolution_Calendar_AlarmNotify__fini,
   ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_AlarmNotify__vepv, GNOME_Evolution_Calendar_AlarmNotify_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_GNOME_Evolution_Calendar_AlarmNotify__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_GNOME_Evolution_Calendar_AlarmNotify__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

