#ifndef __SUNONE_I18N_H__
#define __SUNONE_I18N_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
                                                                                                                                                                                                                 
#ifdef ENABLE_NLS
#include <libintl.h>
#endif

#endif /* __SUNONE_I18N_H__ */

