
#ifndef __sunone_marshal_MARSHAL_H__
#define __sunone_marshal_MARSHAL_H__

#include	<glib-object.h>

G_BEGIN_DECLS

/* NONE:POINTER (./sunone-marshal.list:1) */
#define sunone_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER
#define sunone_marshal_NONE__POINTER	sunone_marshal_VOID__POINTER

G_END_DECLS

#endif /* __sunone_marshal_MARSHAL_H__ */

